@@ -22,7 +22,8 @@
 import sys
 from gi.repository import GObject
 from ..overrides import override
-from ..importer import modules
+from ..module import get_introspection_module
+from gi import PyGIDeprecationWarning
 
 if sys.version_info >= (3, 0):
     _basestring = str
@@ -31,7 +32,8 @@ else:
     _basestring = basestring
     _callable = callable
 
-Gtk = modules['Gtk']._introspection_module
+Gtk = get_introspection_module('Gtk')
+
 __all__ = []
 
 if Gtk._version == '2.0':
@@ -441,7 +443,7 @@ class Dialog(Gtk.Dialog, Container):
         if hasattr(Gtk.DialogFlags, "NO_SEPARATOR") and (flags & Gtk.DialogFlags.NO_SEPARATOR):
             self.set_has_separator(False)
             import warnings
-            warnings.warn("Gtk.DialogFlags.NO_SEPARATOR has been depricated since Gtk+-3.0", DeprecationWarning)
+            warnings.warn("Gtk.DialogFlags.NO_SEPARATOR has been depricated since Gtk+-3.0", PyGIDeprecationWarning)
 
         if buttons is not None:
             self.add_buttons(*buttons)
@@ -492,7 +494,7 @@ class MessageDialog(Gtk.MessageDialog, Dialog):
         # type keyword is used for backwards compat with PyGTK
         if 'type' in kwds:
             import warnings
-            warnings.warn("The use of the keyword type as a parameter of the Gtk.MessageDialog constructor has been depricated. Please use message_type instead.", DeprecationWarning)
+            warnings.warn("The use of the keyword type as a parameter of the Gtk.MessageDialog constructor has been depricated. Please use message_type instead.", PyGIDeprecationWarning)
             message_type = kwds.pop('type')
 
         Gtk.MessageDialog.__init__(self,
@@ -1384,6 +1386,13 @@ class TreeView(Gtk.TreeView, Container):
             path = TreePath(path)
         return super(TreeView, self).get_cell_area(path, column)
 
+    def insert_column_with_attributes(self, position, title, cell, **kwargs):
+        column = TreeViewColumn()
+        column.set_title(title)
+        column.pack_start(cell, False)
+        self.insert_column(column, position)
+        column.set_attributes(cell, **kwargs)
+
 TreeView = override(TreeView)
 __all__.append('TreeView')
 
