@@ -26,10 +26,8 @@
 
 G_BEGIN_DECLS
 
-static PyTypeObject *_PyGObject_Type;
 static PyTypeObject *_PyGTypeWrapper_Type;
 
-#define PyGObject_Type (*_PyGObject_Type)
 #define PyGTypeWrapper_Type (*_PyGTypeWrapper_Type)
 
 __attribute__ ( (unused))
@@ -45,7 +43,7 @@ _pygobject_import (void)
         return 1;
     }
 
-    from_list = Py_BuildValue ("(ss)", "GObject", "GType");
+    from_list = Py_BuildValue ("(s)", "GType");
     if (from_list == NULL) {
         return -1;
     }
@@ -58,12 +56,6 @@ _pygobject_import (void)
         return -1;
     }
 
-    _PyGObject_Type = (PyTypeObject *) PyObject_GetAttrString (module, "GObject");
-    if (_PyGObject_Type == NULL) {
-        retval = -1;
-        goto out;
-    }
-
     _PyGTypeWrapper_Type = (PyTypeObject *) PyObject_GetAttrString (module, "GType");
     if (_PyGTypeWrapper_Type == NULL) {
         retval = -1;
